#import "@preview/cetz:0.3.2": draw,
#import "../element.typ"
#import "../ports.typ": add-port

#let draw-shape(
  id,
  tl,
  tr,
  br,
  bl,
  fill,
  stroke,
  vertical: false,
) = {
  let (x0, y0) = tl
  let (x1, y1) = br

  let w = x1 - x0
  let h = y1 - y0

  let cx = x0 + w / 2
  let cy = y0 + h / 2

  let line = draw.line.with(stroke: stroke)
  let arc = draw.arc.with(stroke: stroke)
  let loops = 4 // Number of half-circle loops

  let f = draw.group(
    name: id,
    {
      if vertical {
        // Vertical Inductor
        let p_start = (cx, y0)
        let p_end = (cx, y1)
        let coil_margin = 0.1 * h // Margin for connection lines
        let coil_start_y = y0 + coil_margin
        let coil_end_y = y1 - coil_margin
        let coil_length = coil_end_y - coil_start_y
        let loop_height = coil_length / loops
        let loop_radius = w / 2 // Radius of the arcs depends on the element's width
        let first_loop_start_y = coil_start_y
        let last_loop_end_y = coil_end_y

        // Draw connection lines (ending at the start/end of the coil loops)
        line(p_start, (cx, first_loop_start_y))
        line((cx, last_loop_end_y), p_end)

        // Draw loops
        for i in range(loops) {
          let loop_center_y = coil_start_y + loop_height * i + loop_height / 2
          arc(
            (cx, loop_center_y),
            start: -90deg,
            delta: 180deg,
            radius: loop_radius,
          )
        }
      } else {
        // Horizontal Inductor
        let p_start = (x0, cy)
        let p_end = (x1, cy)
        let coil_margin = 0.1 * w // Margin for connection lines
        let coil_start_x = x0 + coil_margin
        let coil_end_x = x1 - coil_margin
        let coil_length = coil_end_x - coil_start_x
        let loop_width = coil_length / loops
        let loop_radius = h / 2 // Radius of the arcs depends on the element's height
        let first_loop_start_x = coil_start_x
        let last_loop_end_x = coil_end_x

        draw.merge-path({
          // Draw connection lines (ending at the start/end of the coil loops)
          line(p_start, (first_loop_start_x, cy))

          // Draw loops
          for i in range(loops) {
            let loop_center_x = coil_start_x + loop_width * i + loop_width / 2
            arc(
              (loop_center_x, cy),
              start: 0deg,
              delta: 180deg,
              radius: loop_radius / 2.5,
              anchor: "origin",
            )
          }

          line((last_loop_end_x, cy), p_end)
        })
      }
    },
  )

  return (f, tl, tr, br, bl)
}

#let inductor(
  x: none,
  y: none,
  w: none,
  h: none,
  name: none,
  name-anchor: "center",
  vertical: false,
  fill: none,
  stroke: black + 1pt,
  id: "",
  debug: (
    ports: false,
  ),
) = {
  let ports = if vertical {
    (
      north: ((id: "0"),),
      south: ((id: "1"),),
    )
  } else {
    (
      west: ((id: "0"),),
      east: ((id: "1"),),
    )
  }

  element.elmt(
    draw-shape: draw-shape.with(vertical: vertical),
    x: x,
    y: y,
    w: w,
    h: h,
    name: name,
    name-anchor: name-anchor,
    ports: ports,
    fill: fill,
    stroke: stroke,
    id: id,
    debug: debug,
  )
}
