#import "@preview/cetz:0.3.2": draw
#import "../element.typ"
#import "../ports.typ": add-port

#let draw-shape(
  id, tl, tr, br, bl,
  fill, stroke,
  vertical: false
) = {
  let (x0, y0) = tl
  let (x1, y1) = br

  let w = x1 - x0
  let h = y1 - y0

  let line = draw.line.with(stroke: stroke)
  let cx = x0 + w / 2
  let cy = y0 + h / 2

  let f = draw.group(name: id, {
    if vertical {
      // Vertical voltage source (battery symbol) - Centered
      let p0 = (cx, y0) // North port (+)
      let p1 = (cx, y1) // South port (-)
      let gap = 0.1 * h    // Gap between the two plates
      let plate_width = 0.6 * w // Width of the plates

      // Center the plates around cy
      let plate0_y = cy - gap / 2 // Position of the long plate (+)
      let plate1_y = cy + gap / 2 // Position of the short plate (-)

      // Create a thicker stroke for the short plate
      let thicker_stroke = (paint: stroke.paint, thickness: stroke.thickness + 1pt)

      // Draw plates
      line((cx - plate_width / 2, plate0_y), (cx + plate_width / 2, plate0_y)) // Long plate (+)
      line((cx - plate_width / 3, plate1_y), (cx + plate_width / 3, plate1_y)) // Short plate (-)

      // Draw connection lines (ending at the plates)
      line(p0, (cx, plate0_y)) // Line from North port to top plate
      line((cx, plate1_y), p1) // Line from bottom plate to South port

    } else {
      // Horizontal voltage source (battery symbol) - Centered
      let p0 = (x0, cy) // West port (+)
      let p1 = (x1, cy) // East port (-)
      let gap = 0.1 * w    // Gap between the two plates
      let plate_height = 0.6 * h // Height of the plates

      // Center the plates around cx
      let plate0_x = cx - gap / 2 // Position of the long plate (+)
      let plate1_x = cx + gap / 2 // Position of the short plate (-)

      // Create a thicker stroke for the short plate
      let thicker_stroke = (paint: stroke.paint, thickness: stroke.thickness + 1pt)
      
      // Draw plates
      line((plate0_x, cy - plate_height / 2), (plate0_x, cy + plate_height / 2)) // Long plate (+)
      line((plate1_x, cy - plate_height / 3), (plate1_x, cy + plate_height / 3)) // Short plate (-)

      // Draw connection lines (ending at the plates)
      line(p0, (plate0_x, cy)) // Line from West port to left plate
      line((plate1_x, cy), p1) // Line from right plate to East port
    }
  })
  
  return (f, tl, tr, br, bl)
}

#let voltage-source(
  x: none,
  y: none,
  w: none,
  h: none,
  name: none,
  name-anchor: "center",
  vertical: false,
  fill: none,
  stroke: black + 1pt,
  id: "",
  debug: (
    ports: false
  )
) = {
  let ports = if vertical {(
    north: ((id: "0"),),
    south: ((id: "1"),)
  )} else {(
    west: ((id: "0"),),
    east: ((id: "1"),)
  )}
  
  element.elmt(
    draw-shape: draw-shape.with(
      vertical: vertical
    ),
    x: x, y: y, w: w, h: h, name: name, name-anchor: name-anchor,
    ports: ports, fill: fill, stroke: stroke, id: id, debug: debug
  )
}